/***************************************************************************
 *   Copyright 2013 Sebastian Kügler <sebas@kde.org>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU Library General Public License for more details.                  *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

.pragma library

function alignedSize(suggestedsize) {

    var resultingsize = suggestedsize;

    if (suggestedsize < 16) {
        //do nothing
    } else if (suggestedsize < 22) {
        resultingsize = 16;
    } else if (suggestedsize < 24) {
        resultingsize = 22;
    } else if (suggestedsize < 32) {
        resultingsize = 24;
    } else if (suggestedsize < 48) {
        resultingsize = 32;
    } else if (suggestedsize < 64) {
        resultingsize = 48;
        //if size is more than 64, leave as is
    }

    return resultingsize;
}


